#include <iostream>
#include <iomanip>
#include <cstddef>
#include <cstring>

struct Student {
    char name[17];
    uint16_t year;
    float gpa;
    unsigned gender : 1;
    uint8_t courses;
    Student* leader;
};

void printStudentInfo(const Student &student) {
    std::cout << ": " << student.name << std::endl;
    std::cout << " : " << student.year << std::endl;
    std::cout << " : " << student.gpa << std::endl;
    std::cout << ": " << (student.gender ? "" : "") << std::endl;
    std::cout << "  : " << static_cast<int>(student.courses) << std::endl;
    std::cout << "  : " << student.leader << std::endl;
}

void printMemoryLayout(const Student &student) {
    std::cout << " : " << &student << std::endl;
    std::cout << " : " << offsetof(Student, name) << ", : " << sizeof(student.name) << std::endl;
    std::cout << "  : " << offsetof(Student, year) << ", : " << sizeof(student.year) << std::endl;
    std::cout << "  : " << offsetof(Student, gpa) << ", : " << sizeof(student.gpa) << std::endl;
    //    offsetof  gender
    // std::cout << " : " << offsetof(Student, gender) << ", : 1 " << std::endl;
    std::cout << ":  1  (     offset-of)" << std::endl;
    std::cout << " : " << offsetof(Student, courses) << ", : " << sizeof(student.courses) << std::endl;
    std::cout << "   : " << offsetof(Student, leader) << ", : " << sizeof(student.leader) << std::endl;

    std::cout << "   : ";
    const unsigned char* bytes = reinterpret_cast<const unsigned char*>(&student);
    for (size_t i = 0; i < sizeof(Student); ++i) {
        std::cout << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(bytes[i]) << " ";
    }
    std::cout << std::dec << std::endl;
}

int main() {
    Student student1 = {"Alice", 2021, 3.8, 0, 5, nullptr};
    Student student2 = {"Bob", 2021, 3.5, 1, 6, &student1};
    Student starosta = {"Charlie", 2020, 4.0, 1, 8, nullptr};

    Student students[] = {student1, student2, starosta};

    std::cout << " : " << &students << ", : " << sizeof(students) << std::endl;

    for (size_t i = 0; i < 3; ++i) {
        std::cout << "\n---  " << i + 1 << " ---" << std::endl;
        printStudentInfo(students[i]);
        printMemoryLayout(students[i]);
    }

    return 0;
}
